#!/usr/bin/python

"""
Conch
A sound and music toolkit.
Builds on the Pygame toolkit to make loading and playing sounds
and music slightly easier.
"""

__author__ = "Kris Schnee"
__version__ = "2008.1.7"
__license__ = "Public Domain"

import pygame ## Pygame toolkit for sound (and many other things); pygame.org
import os     ## File system

music = True
sound = True
DEFAULT_MUSIC_EXTENSION = ".mid"
DEFAULT_SOUND_EXTENSION = ".wav"
SOUND_DIRECTORY = os.path.join("sound","effects")
MUSIC_DIRECTORY = os.path.join("sound","music")

current_song = None ## For LoopSong

pygame.mixer.init()

songs = {}
sounds = {}

def ToggleMusic(on=True):
    music = on
def ToggleSound(on=True):
    sound = on

def StopMusic():
    pygame.mixer.music.stop()

def LoadSong(songname,key):
    """Add name, including directory location, to self.songs.
    You can give the song a key for easy reference.
    Note that rather than actually loading the song, I store only
    the path to it. Contrast with sound loading."""
    new_song_path = os.path.join(MUSIC_DIRECTORY,songname)
    songs[ key ] = new_song_path

def LoopSong():
    if not current_song:
        raise Exception("There is no currently playing song to loop.")
    self.PlaySong(current_song)

def PlaySong(cue_name,interrupt=False):
    path = songs.get( cue_name )
    print "a"
    if path:
        print "b"
        if not music:
            return ## Never mind.
        print "c"
        if not interrupt:
            if pygame.mixer.music.get_busy():
                return ## It's busy; go away.

        ## OK, we can play. First stop whatever's playing.
        pygame.mixer.music.stop()

        ## Now load and play.
        try:
            pygame.mixer.music.load(path)
        except:
            print "Couldn't load song '"+cue_name+"'."
            return
        try:
            pygame.mixer.music.play()
            current_song = cue_name
        except:
            print "Couldn't play song '"+cue_name+"'."

def LoadSound(filename,key):
    """Load a sound into memory, not just its name, for quick use."""
    if not "." in filename:
        filename += DEFAULT_SOUND_EXTENSION
    new_sound = pygame.mixer.Sound( os.path.join(SOUND_DIRECTORY,filename) )
    sounds[ key ] = new_sound

def PlaySound(key):
    ## How to check whether sound player is busy?
    if sounds.has_key( key ):
        sounds[ key ].play()
    else:
        print "Couldn't play sound \""+key+"\", but it's not loaded."
